@@ -1,26 +1,47 @@
 import re
 import sys
 from time import sleep
-from typing import Any
 
 import requests
 
-import mangadlp.utils as utils
+from mangadlp import utils
+from mangadlp.logger import Logger
+
+# prepare logger
+log = Logger(__name__)
 
 
 class Mangadex:
+    """Mangadex API Class.
+    Get infos for a manga from mangadex.org
+
+    Args:
+        url_uuid (str): URL or UUID of the manga
+        language (str): Manga language with country codes. "en" --> english
+        forcevol (bool): Force naming of volumes. Useful for mangas where chapters reset each volume
+
+    Attributes:
+        api_name (str): Name of the API
+        manga_uuid (str): UUID of the manga, without the url part
+        manga_data (dict): Infos of the manga. Name, title etc
+        manga_title (str): The title of the manga, sanitized for all filesystems
+        manga_chapter_data (dict): All chapter data of the manga. Volumes, chapters, chapter uuids and chapter names
+        chapter_list (list): A list of all available chapters for the language
+
+    """
 
     # api information
     api_base_url = "https://api.mangadex.org"
     img_base_url = "https://uploads.mangadex.org"
 
     # get infos to initiate class
-    def __init__(self, url_uuid: str, language: str, forcevol: bool, verbosity: int):
+    def __init__(self, url_uuid: str, language: str, forcevol: bool):
         # static info
+        self.api_name = "Mangadex"
+
         self.url_uuid = url_uuid
         self.language = language
         self.forcevol = forcevol
-        self.verbosity = verbosity
 
         # api stuff
         self.api_content_ratings = "contentRating[]=safe&contentRating[]=suggestive&contentRating[]=erotica&contentRating[]=pornographic"
@@ -35,91 +56,90 @@ class Mangadex:
         self.chapter_list = self.create_chapter_list()
 
     # make initial request
-    def get_manga_data(self) -> requests.Response:
-        if self.verbosity >= 2:
-            print(f"INFO: Getting manga data for: {self.manga_uuid}")
+    def get_manga_data(self) -> dict:
+        log.verbose(f"Getting manga data for: {self.manga_uuid}")
         counter = 1
         while counter <= 3:
             try:
                 manga_data = requests.get(
                     f"{self.api_base_url}/manga/{self.manga_uuid}"
                 )
-            except:
+            except Exception:
                 if counter >= 3:
-                    print("ERR: Maybe the MangaDex API is down?")
+                    log.error("Maybe the MangaDex API is down?")
                     sys.exit(1)
                 else:
-                    print("ERR: Mangadex API not reachable. Retrying")
+                    log.error("Mangadex API not reachable. Retrying")
                     sleep(2)
                     counter += 1
             else:
                 break
         # check if manga exists
         if manga_data.json()["result"] != "ok":
-            print("ERR: Manga not found")
+            log.error("Manga not found")
             sys.exit(1)
 
-        return manga_data
+        return manga_data.json()
 
     # get the uuid for the manga
     def get_manga_uuid(self) -> str:
         # isolate id from url
-        uuid_regex: Any = re.compile(
+        uuid_regex = re.compile(
             "[a-z0-9]{8}-[a-z0-9]{4}-[a-z0-9]{4}-[a-z0-9]{4}-[a-z0-9]{12}"
         )
-        # check for new mangadex id
-        if not uuid_regex.search(self.url_uuid):
-            print("ERR: No valid UUID found")
+        # try to get uuid in string
+        try:
+            uuid = uuid_regex.search(self.url_uuid)[0]  # type: ignore
+        except Exception:
+            log.error("No valid UUID found")
             sys.exit(1)
-        manga_uuid = uuid_regex.search(self.url_uuid)[0]
-        return manga_uuid
+
+        return uuid
 
     # get the title of the manga (and fix the filename)
     def get_manga_title(self) -> str:
-        if self.verbosity >= 2:
-            print(f"INFO: Getting manga title for: {self.manga_uuid}")
-        manga_data = self.manga_data.json()
+        log.verbose(f"Getting manga title for: {self.manga_uuid}")
+        manga_data = self.manga_data
         try:
             title = manga_data["data"]["attributes"]["title"][self.language]
-        except:
+        except Exception:
             # search in alt titles
             try:
                 alt_titles = {}
                 for title in manga_data["data"]["attributes"]["altTitles"]:
                     alt_titles.update(title)
                 title = alt_titles[self.language]
-            except:  # no title on requested language found
-                print("ERR: Chapter in requested language not found.")
+            except Exception:  # no title on requested language found
+                log.error("Chapter in requested language not found.")
                 sys.exit(1)
+
         return utils.fix_name(title)
 
     # check if chapters are available in requested language
     def check_chapter_lang(self) -> int:
-        if self.verbosity >= 2:
-            print(
-                f"INFO: Checking for chapters in specified language for: {self.manga_uuid}"
-            )
+        log.verbose(
+            f"Checking for chapters in specified language for: {self.manga_uuid}"
+        )
         r = requests.get(
             f"{self.api_base_url}/manga/{self.manga_uuid}/feed?limit=0&{self.api_additions}"
         )
         try:
             total_chapters = r.json()["total"]
-        except:
-            print(
-                "ERR: Error retrieving the chapters list. Did you specify a valid language code?"
+        except Exception:
+            log.error(
+                "Error retrieving the chapters list. Did you specify a valid language code?"
             )
             return 0
         else:
             if total_chapters == 0:
-                print("ERR: No chapters available to download!")
+                log.error("No chapters available to download!")
                 return 0
 
         return total_chapters
 
     # get chapter data like name, uuid etc
     def get_chapter_data(self) -> dict:
-        if self.verbosity >= 2:
-            print(f"INFO: Getting chapter data for: {self.manga_uuid}")
+        log.verbose(f"Getting chapter data for: {self.manga_uuid}")
         api_sorting = "order[chapter]=asc&order[volume]=asc"
         # check for chapters in specified lang
         total_chapters = self.check_chapter_lang()
@@ -177,8 +197,7 @@ class Mangadex:
 
     # get images for the chapter (mangadex@home)
     def get_chapter_images(self, chapter: str, wait_time: float) -> list:
-        if self.verbosity >= 2:
-            print(f"INFO: Getting chapter images for: {self.manga_uuid}")
+        log.verbose(f"Getting chapter images for: {self.manga_uuid}")
         athome_url = f"{self.api_base_url}/at-home/server"
         chapter_uuid = self.manga_chapter_data[chapter][0]
 
@@ -190,20 +209,20 @@ class Mangadex:
                 r = requests.get(f"{athome_url}/{chapter_uuid}")
                 api_data = r.json()
                 if api_data["result"] != "ok":
-                    print(f"ERR: No chapter with the id {chapter_uuid} found")
+                    log.error(f"No chapter with the id {chapter_uuid} found")
                     api_error = True
                     raise IndexError
-                elif api_data["chapter"]["data"] is None:
-                    print(f"ERR: No chapter data found for chapter {chapter_uuid}")
+                if api_data["chapter"]["data"] is None:
+                    log.error(f"No chapter data found for chapter {chapter_uuid}")
                     api_error = True
                     raise IndexError
-                else:
-                    api_error = False
-                    break
-            except:
+                # no error
+                api_error = False
+                break
+            except Exception:
                 if counter >= 3:
                     api_error = True
-                print(f"ERR: Retrying in a few seconds")
+                log.error("Retrying in a few seconds")
                 counter += 1
                 sleep(wait_time + 2)
         # check if result is ok
@@ -220,12 +239,12 @@ class Mangadex:
             image_urls.append(f"{self.img_base_url}/data/{chapter_hash}/{image}")
 
         sleep(wait_time)
+
         return image_urls
 
     # create list of chapters
     def create_chapter_list(self) -> list:
-        if self.verbosity >= 2:
-            print(f"INFO: Creating chapter list for: {self.manga_uuid}")
+        log.verbose(f"Creating chapter list for: {self.manga_uuid}")
         chapter_list = []
         for chapter in self.manga_chapter_data.items():
             chapter_info = self.get_chapter_infos(chapter[0])
@@ -240,10 +259,7 @@ class Mangadex:
 
     # create easy to access chapter infos
     def get_chapter_infos(self, chapter: str) -> dict:
-        if self.verbosity >= 3:
-            print(
-                f"INFO: Getting chapter infos for: {self.manga_chapter_data[chapter][0]}"
-            )
+        log.debug(f"Getting chapter infos for: {self.manga_chapter_data[chapter][0]}")
         chapter_uuid = self.manga_chapter_data[chapter][0]
         chapter_vol = self.manga_chapter_data[chapter][1]
         chapter_num = self.manga_chapter_data[chapter][2]
